Package: agtboost
Type: Package
Title: Adaptive and Automatic Gradient Boosting Computations
Version: 0.9.3
Date: 2021-11-23
Author: Berent Ånund Strømnes Lunde
Maintainer: Berent Ånund Strømnes Lunde <lundeberent@gmail.com>
Description: Fast and automatic gradient tree boosting designed
    to avoid manual tuning and cross-validation by utilizing an information
    theoretic approach. This makes the algorithm adaptive to the dataset at
    hand; it is completely automatic, and with minimal worries of overfitting.
    Consequently, the speed-ups relative to state-of-the-art implementations
    can be in the thousands while mathematical and technical knowledge required
    on the user are minimized.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.6.0)
Imports: methods, Rcpp (>= 1.0.1)
LinkingTo: Rcpp, RcppEigen
RcppModules: aGTBModule
RoxygenNote: 7.1.2
Suggests: testthat
NeedsCompilation: yes
Packaged: 2021-11-23 20:53:03 UTC; berentanundstromneslunde
Repository: CRAN
Date/Publication: 2021-11-23 21:10:02 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-24 03:36:18 UTC; unix
Archs: agtboost.so.dSYM
