# Generated by roxygen2: do not edit by hand

S3method(predict,Rcpp_ENSEMBLE)
S3method(predict,Rcpp_GBT_COUNT_AUTO)
export(gbt.complexity)
export(gbt.convergence)
export(gbt.importance)
export(gbt.ksval)
export(gbt.load)
export(gbt.save)
export(gbt.train)
import(Rcpp)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(methods,new)
importFrom(stats,dbinom)
importFrom(stats,dgamma)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,ks.test)
importFrom(stats,pbinom)
importFrom(stats,pgamma)
importFrom(stats,pnbinom)
importFrom(stats,pnorm)
importFrom(stats,ppois)
importFrom(stats,runif)
useDynLib(agtboost, .registration = TRUE)
